#!/usr/bin/env python

# Copyright (C) 2002 Jrg Lehmann <joerg@luga.de>
#
# This file is part of PyTone (http://www.luga.de/pytone/)
#
# PyTone is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2
# as published by the Free Software Foundation.
#
# PyTone is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyX; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import exceptions
import Queue

from helper import debug


class TerminateEventProcessing(exceptions.Exception):
    pass


class event:
    def __str__(self):
        return self.__class__.__name__

class keypressed(event):
    def __init__(self, key):
        self.key = key
        
    def __str__(self):
        return "%s(%d)" % (self.__class__.__name__, self.key)


class selectionchanged(event):
    def __init__(self, item):
        self.item = item

    def __str__(self):
        return "%s(%s)" % (self.__class__.__name__, `self.item`)


class activewinchanged(event):
    def __init__(self, active):
        self.active = active

    def __str__(self):
        return "%s(%s)" % (self.__class__.__name__, `self.active`)


class activateplaylist(event):
    pass


class activatefilelist(event):
    pass


class artistaddedtosongdb(event):
    def __init__(self, artist):
        self.artist = artist

    def __str__(self):
        return "%s(%s)" % (self.__class__.__name__, `self.artist`)

        
class albumaddedtosongdb(event):
    def __init__(self, album):
        self.album = album

    def __str__(self):
        return "%s(%s)" % (self.__class__.__name__, `self.album`)
    

class requestnextsong(event):
    def __init__(self, player):
        self.player = player
        
    def __str__(self):
        return "%s(%s)" % (self.__class__.__name__, `self.player`)


class nextsong(event):
    def __init__(self, player, song):
        self.player = player
        self.song = song
        
    def __str__(self):
        return "%s(%s->%s)" % (self.__class__.__name__, `self.song`, `self.player`)


class playerstart(event):
    def __init__(self, player):
        self.player = player
        
    def __str__(self):
        return "%s(%s)" % (self.__class__.__name__, `self.player`)


class playerforward(event):
    def __init__(self, player):
        self.player = player
        
    def __str__(self):
        return "%s(%s)" % (self.__class__.__name__, `self.player`)


class playerstop(event):
    def __init__(self, player):
        self.player = player
        
    def __str__(self):
        return "%s(%s)" % (self.__class__.__name__, `self.player`)


class playbackinfochanged(event):
    def __init__(self, player, playing=0, title=None, ptime=None, ttime=None):
        self.player = player
        self.playing = playing
        self.title = title
        self.ptime = ptime
        self.ttime = ttime
        
    def __str__(self):
        if self.playing:
            return "%s(%s playing %s %d/%d )" % (self.__class__.__name__,
                                                 `self.player`,
                                                 self.title,
                                                 self.ptime,
                                                 self.ttime)
        else:
            return "%s(%s stop)" % (self.__class__.__name__,
                                    `self.player`)
        
class updatestatusbar:
    def __init__(self, content):
        self.content = content

    def __str__(self):
        return "%s(%s)" % (self.__class__.__name__, self.content)


class requestinput:
    def __init__(self, prompt, handler):
        self.prompt = prompt
        self.handler = handler

    def __str__(self):
        return "%s(%s,%s)" % (self.__class__.__name__, self.prompt, `self.handler`)


class songupdate:
    def __init__(self, song):
        self.song = song
        
    def __str__(self):
        return "%s(%s)" % (self.__class__.__name__, `self.song`)


class playlistaddsong(event):
    def __init__(self, song):
        self.song = song
        
    def __str__(self):
        return "%s(%s)" % (self.__class__.__name__, `self.song`)


class playlistload(event):
    pass


class playlistsave(event):
    pass


class playlistclear(event):
    pass


class playlistdeleteplayedsongs(event):
    pass


#
# event dispatcher classes
#

class eventhub:

    """ collects event channels from different threads """
    
    def __init__(self):
        self.channels = []

    def connect(self, channel):
        self.channels.append(channel)

    def newchannel(self):
        channel = eventchannel()
        self.connect(channel)
        return channel

    def notify(self, event):
        debug("eventhub: %s\n" % str(event))
        for channel in self.channels:
            channel.notify(event)
            

class eventchannel:

    """ collects event handlers for one thread """
    
    def __init__(self):
        self.subscriptions = []
        self.queue = Queue.Queue(-1)

    def process(self):
        while not self.queue.empty():
            event = self.queue.get()
            try:
                for subscribedevent, handler in self.subscriptions:
                    if isinstance(event, subscribedevent):
                        handler(event)
            except TerminateEventProcessing:
                pass
                
    def subscribe(self, eventtype, handler):
        self.subscriptions.append((eventtype, handler))

    def unsubscribe(self, eventtype, handler):
        self.subscriptions.remove((eventtype, handler))
        
    def notify(self, event):
        self.queue.put(event)

hub = eventhub()
