#!/usr/bin/env python

# Copyright (C) 2002 Jrg Lehmann <joerg@luga.de>
#
# This file is part of PyTone (http://www.luga.de/pytone/)
#
# PyTone is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2
# as published by the Free Software Foundation.
#
# PyTone is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyX; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os.path
import random

import events
import item
import slist

from helper import debug

class filelistitem:
    def __init__(self, item):
        self.item = item

    def __repr__(self):
        return `self.item`

    def __getattr__(self, attr):
        return self.item.__dict__[attr]

    def isdir(self):
        return isinstance(self.item, item.artist) or \
               isinstance(self.item, item.album)


class filelist(slist.slist):
    def __init__(self, win, mp3basedir, songdb):
        slist.slist.__init__(self, win.ih)
        self.win = win
        self.mp3basedir = mp3basedir
        self.songdb = songdb
        self.shistory = []
        
        # we assume that we are active upon initialization
        self.active = 1
        
        self.readdir()

    def getselected(self):
        filelistitem = slist.slist.getselected(self)
        if filelistitem:
            return filelistitem.item
        else:
            return None

    def isdirselected(self):
        return slist.slist.getselected(self).isdir()

    def dirdown(self):
        self.shistory.append((self.dir, self.items, self.selected, self.top))
        self.dir = self.getselectedsubdir()
        
        if self.readdir() is None:
            self.dir, self.items, self.selected, self.top = self.shistory.pop()

    def dirup(self):
        if len(self.shistory)>0:
            dir, items, selected, top = self.shistory.pop()
            self.dir = dir
            if dir is not None:
                self.readdir()
                self.selected = selected
                self.top = top
            events.hub.notify(events.selectionchanged(self.getselected()))

    def generaterandomlist(self):
        """generate random list of songs in songdb filling filelistwin"""
        songlist = self.songdb.getsongs()
        
        if len(songlist)==0:
            return
        
        if self.dir is not None:
            self.shistory.append((self.dir, self.items, self.selected, self.top))

        self.clear()
        self.dir = None

        for i in range(self.height):
            self.append(filelistitem(random.choice(songlist)))
            

class dirbasedfilelist(filelist):

    def __init__(self, win, mp3basedir, songdb):
        self.dir = mp3basedir
        filelist.__init__(self, win, mp3basedir, songdb)
        
    def getselectedsubdir(self):
        return self.getselected().path
    
    def readdir(self):
        self.clear()
        
        assert os.path.isdir(self.dir)
        try:
            for name in os.listdir(self.dir):
                try:
                    path = os.path.join(self.dir, name)
                    if os.path.isdir(path) and os.access(path, os.R_OK|os.X_OK):
                        os.listdir(path)
                        subdirs = path[len(self.mp3basedir):].split("/")[1:]
                        if len(subdirs)==1:
                            newitem = item.artist(subdirs[0], path)
                        else:
                            newitem = item.album(subdirs[0], subdirs[1], path)
                        self.append(filelistitem(newitem))
                    elif name.endswith(".mp3") and os.access(path, os.R_OK):
                        newsong = self.songdb.queryregister(path)
                        self.append(filelistitem(newsong))
                except (IOError, OSError) : pass
        except OSError:
            return None

        self.sort()
        return 1

    def insertrecursiveselection(self):
        self.insertrecursive(self.getselected().path)

    def insertrecursive(self, dir):
        try:
            list = os.listdir(dir)
            list.sort()
        except:
            return
        for name in list:
            try:
                path = os.path.join(dir, name)
                if os.path.isdir(path):
                    self.insertrecursive(path)
                elif name[-4:]==".mp3":
                    events.hub.notify(events.playlistaddsong(item.song(path)))
            except (IOError, OSError): pass

    def selectionpath(self):
        if self.dir is not None:
            if self.dir==self.mp3basedir:
                return "MP3s"
            else:
                return self.dir[len(self.mp3basedir)+1:]
        else:
            return self.getselected().artist + " - " + self.getselected().album

class dbbasedfilelist(filelist):

    def __init__(self, win, mp3basedir, songdb):
        self.dir = []
        filelist.__init__(self, win, mp3basedir, songdb)
        self.win.channel.subscribe(events.artistaddedtosongdb,
                                   self.artistaddedtosongdb)
        self.win.channel.subscribe(events.albumaddedtosongdb,
                                   self.albumaddedtosongdb)

    def __del__(self):
        if events:
            self.win.channel.unsubscribe(events.artistaddedtosongdb,
                                         self.artistaddedtosongdb)
            self.win.channel.unsubscribe(events.artistaddedtosongdb,
                                         self.artistaddedtosongdb)
        
    def getselectedsubdir(self):
        return self.dir + [self.getselected()]

    def readdir(self):
        self.clear()

        if not self.dir:
            for artist in self.songdb.getartists():
                self.append(filelistitem(artist))
        elif isinstance(self.dir[-1], item.artist):
            artist = self.dir[-1]
            for album in self.songdb.getalbums(artist.name):
                self.append(filelistitem(album))
        elif isinstance(self.dir[-1], item.album):
            album = self.dir[-1]
            for song in self.songdb.getsongs(album.artist, album.name):
                self.append(filelistitem(song))
        else:
            assert 0, "internal programm error"

        self.sort()
            
        return 1


    def selectionpath(self):
        if self.dir is not None:
            if len(self.dir)==0:
                return "MP3 Database (%d entries)" % len(self.songdb.songs)
            elif len(self.dir)==1:
                return self.getselected().artist
            else:
                return self.getselected().artist + " - " + self.getselected().album
        else:
            return self.getselected().artist + " - " + self.getselected().album

    def insertrecursiveselection(self):
        
        if isinstance(self.getselected(), item.artist):
            insertlist = self.songdb.getsongs(self.getselected().name)

        elif isinstance(self.getselected(), item.album):
            insertlist = self.songdb.getsongs(self.getselected().artist,
                                              self.getselected().name)

        insertlist.sort(lambda x,y: cmp(x.path,y.path))

        for song in insertlist:
            events.hub.notify(events.playlistaddsong(song))

    # event handler

    def artistaddedtosongdb(self, event):
        if self.dir is not None and len(self.dir)==0:
            self.insertitem(filelistitem(event.artist))
            self.win.update()

    def albumaddedtosongdb(self, event):
        if self.dir is not None and len(self.dir)==1 and \
           self.dir[0].name==event.album.artist:
            self.insertitem(filelistitem(event.album))
            self.win.update()
